import fnmatch
import os

from django.contrib.staticfiles import finders
from django.contrib.staticfiles.storage import staticfiles_storage


def find_all_files(glob):
    """
    Finds all files in the django finders for a given glob,
    returns the file path, if available, and the django storage object.
    storage objects must implement the File storage API:
    https://docs.djangoproject.com/en/dev/ref/files/storage/
    """
    for finder in finders.get_finders():
        for path, storage in finder.list([]):
            if fnmatch.fnmatchcase(os.path.join(getattr(storage, 'prefix', '')
                                                or '', path),
                                   glob):
                yield path, storage


def get_file_from_storage(filename):
    try:
        filename = staticfiles_storage.path(filename)
    except NotImplementedError:
        # remote storages don't implement path
        pass
    if staticfiles_storage.exists(filename):
        return filename, staticfiles_storage
    else:
        return None, None


def get_file_from_finders(filename):
    for file_and_storage in find_all_files(filename):
        return file_and_storage
    return None, None


def get_file_and_storage(filename):
    name, storage = get_file_from_finders(filename)
    # get_file_from_finders could fail in production if code is a deployed as a
    # package without it's package_data. In that case, we'd assume that
    # collectstatic had been run and we can get the file from storage.
    if storage is None:
        name, storage = get_file_from_storage(filename)
    return name, storage



"""
Django settings for testproject project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))


from django import VERSION as DJANGO_VERSION
if DJANGO_VERSION[:2] < (1, 6):
    TEST_RUNNER = 'discover_runner.DiscoverRunner'


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'testproject'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

TEMPLATE_DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'compressor',

    'testproject.testapp1',
    'testproject.testapp2',
]

if DJANGO_VERSION[:2] < (1, 6):
    INSTALLED_APPS += ('discover_runner', )

MIDDLEWARE = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'testproject.urls'

WSGI_APPLICATION = 'testproject.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(BASE_DIR, '..', 'tmp', 'static')

STATICFILES_DIRS = (
    os.path.join(BASE_DIR, 'testproject', 'static'),
    ('css_prefix',
     os.path.join(BASE_DIR, 'testproject', 'static', 'css', 'prefixed')),
)

# It doesn't seem like django-compressor respects override_settings
COMPRESS_ENABLED = True
COMPRESS_PRECOMPILERS = (
    ('text/x-scss', 'django_pyscss.compressor.DjangoScssFilter'),
)

STATICFILES_FINDERS = (
    'compressor.finders.CompressorFinder',
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

# LOGGING
LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        '': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'INFO',
        },
    }
}



import os
import re
import mock

from django.test import TestCase
from django.conf import settings

from scss.errors import SassImportError

from django_pyscss import DjangoScssCompiler

from tests.utils import clean_css, CollectStaticTestCase, NoCollectStaticTestCase


with open(os.path.join(settings.BASE_DIR, 'testproject', 'static', 'css', 'foo.scss')) as f:
    FOO_CONTENTS = f.read()

with open(os.path.join(settings.BASE_DIR, 'testapp1', 'static', 'css', 'app1.scss')) as f:
    APP1_CONTENTS = f.read()

APP2_CONTENTS = FOO_CONTENTS + APP1_CONTENTS

SASS_CONTENTS = """
.sass {
  color: #009900;
}
"""

with open(os.path.join(settings.BASE_DIR, 'testproject', 'static', 'css', 'css_file.css')) as f:
    CSS_CONTENTS = f.read()

with open(os.path.join(settings.BASE_DIR, 'testproject', 'static', 'css', '_baz.scss')) as f:
    BAZ_CONTENTS = f.read()

with open(os.path.join(settings.BASE_DIR, 'testproject', 'static', 'css', 'sub', '_spam.scss')) as f:
    SPAM_CONTENTS = f.read()

BAZ_CONTENTS = BAZ_CONTENTS.replace('@import "sub/spam";', SPAM_CONTENTS)

with open(os.path.join(settings.BASE_DIR, 'testproject', 'static', 'css', 'path_conflict.scss')) as f:
    PATH_CONFLICT_CONTENTS = f.read()

with open(os.path.join(settings.BASE_DIR, 'testproject', 'static', 'css', 'dot.file.scss')) as f:
    DOT_FILE_CONTENTS = f.read()


class CompilerTestMixin(object):
    def setUp(self):
        self.compiler = DjangoScssCompiler()
        super(CompilerTestMixin, self).setUp()


class ImportTestMixin(CompilerTestMixin):
    def test_import_from_staticfiles_dirs(self):
        actual = self.compiler.compile_string('@import "/css/foo.scss";')
        self.assertEqual(clean_css(actual), clean_css(FOO_CONTENTS))

    def test_import_from_staticfiles_dirs_prefixed(self):
        actual = self.compiler.compile_string('@import "/css_prefix/baz.scss";')
        self.assertEqual(clean_css(actual), clean_css(FOO_CONTENTS))

    def test_import_from_staticfiles_dirs_relative(self):
        actual = self.compiler.compile_string('@import "css/foo.scss";')
        self.assertEqual(clean_css(actual), clean_css(FOO_CONTENTS))

    def test_import_from_app(self):
        actual = self.compiler.compile_string('@import "/css/app1.scss";')
        self.assertEqual(clean_css(actual), clean_css(APP1_CONTENTS))

    def test_import_from_app_relative(self):
        actual = self.compiler.compile_string('@import "css/app1.scss";')
        self.assertEqual(clean_css(actual), clean_css(APP1_CONTENTS))

    def test_imports_within_file(self):
        actual = self.compiler.compile_string('@import "/css/app2.scss";')
        self.assertEqual(clean_css(actual), clean_css(APP2_CONTENTS))

    def test_relative_import(self):
        actual = self.compiler.compile('/css/bar.scss')
        self.assertEqual(clean_css(actual), clean_css(FOO_CONTENTS))

    def test_relative_import_with_filename(self):
        actual = self.compiler.compile_string('@import "foo.scss";', 'css/bar.scss')
        self.assertEqual(clean_css(actual), clean_css(FOO_CONTENTS))

    def test_bad_import(self):
        self.assertRaises(SassImportError, self.compiler.compile_string, '@import "this-file-does-not-and-should-never-exist.scss";')

    def test_no_extension_import(self):
        actual = self.compiler.compile_string('@import "/css/foo";')
        self.assertEqual(clean_css(actual), clean_css(FOO_CONTENTS))

    def test_no_extension_import_sass(self):
        actual = self.compiler.compile_string('@import "/css/sass_file";')
        self.assertEqual(clean_css(actual), clean_css(SASS_CONTENTS))

    # def test_no_extension_import_css(self):
    #     actual = self.compiler.compile_string('@import "/css/css_file";')
    #     self.assertEqual(clean_css(actual), clean_css(CSS_CONTENTS))

    def test_import_underscore_file(self):
        actual = self.compiler.compile_string('@import "/css/baz";')
        self.assertEqual(clean_css(actual), clean_css(BAZ_CONTENTS))

    def test_import_conflict(self):
        actual = self.compiler.compile_string('@import "/css/path_conflict";')
        self.assertEqual(clean_css(actual), clean_css(PATH_CONFLICT_CONTENTS))

    def test_import_dots_without_extension(self):
        actual = self.compiler.compile_string('@import "/css/dot.file";')
        self.assertEqual(clean_css(actual), clean_css(DOT_FILE_CONTENTS))

    def test_import_dots_with_extension(self):
        actual = self.compiler.compile_string('@import "/css/dot.file.scss";')
        self.assertEqual(clean_css(actual), clean_css(DOT_FILE_CONTENTS))

    def test_import_from_parent(self):
        actual = self.compiler.compile_string('@import "/css/sub/from_parent";')
        self.assertEqual(clean_css(actual), clean_css(BAZ_CONTENTS))

class FindersImportTest(ImportTestMixin, NoCollectStaticTestCase):
    pass


# Emulate the condition where collectstatic was run but the source files are no
# longer available.
@mock.patch('django_pyscss.utils.get_file_from_finders',
            new=mock.MagicMock(return_value=(None, None)))
class StorageImportTest(ImportTestMixin, CollectStaticTestCase):
    pass


INLINE_IMAGE = """
.loading {
  background: inline_image('images/loading.gif') center no-repeat;
}
"""

INLINED_IMAGE_EXPECTED = """
.loading {
  background: url(data:image/gif;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAABs0lEQVR4nKXTzUojQRDA8X9/zJCIECRfiDCDBHKZq+QR8nJ5Hl/AQ2QgxyCOoCAy8eIkAxLy0dPWnkzQ7JpdtqAv3fSP6qouJSL8T9jvGy8vLzKdTsmyjDzPGY1G6q+B29tbubm5YTab4b3n4+Njd1ZVlYgIWmuMMeoASNNUrq+vWa/XDAYDkiTh8vJyByil8N7jnCMMQ7HWqh2Q57mMx2OccwyHQ4bD4UHaxhglIuKcY71eU6/XxRijNMDDwwNFUXB1dfXby7t0rVVaa5xzbDYbADTA8/MzWmuSJPmpXp8IVVXtAAtQFAVBEBBF0VEgDENVVZV47/eA1hprLUr92DEAvPfinENrvX/C+fk5QRAwm82OAqvVCuccxpg9EMcxxhienp6OAmVZst1uCcNwD/R6PbrdLo+Pj0wmkz/+7dfXV3l7e8Nay8nJCQDqcxbu7u4kTVNEhH6/TxzHdDodlFKUZclisWA+n1Or1YiiiGazqb4AAPf395JlGe/v71hrCcPwy2o0GlxcXNDpdHbVVt+ncT6fS57nFEXBcrkkCALOzs5otVq0221OT0+/tOoA+Nf4BawEx9v4UlbsAAAAAElFTkSuQmCC) center no-repeat;
}
"""

SPRITE_MAP = """
$widgets: sprite-map('images/icons/widget-*.png');

.foo {
  background: sprite($widgets, 'widget-skull-and-crossbones');
}

.bar {
  background: sprite($widgets, 'widget-google-map');
}
"""


class AssetsTest(CompilerTestMixin, TestCase):
    def test_sprite_images(self):
        actual = self.compiler.compile_string(SPRITE_MAP)
        # pyScss puts a cachebuster query string on the end of the URLs, lets
        # just check that it made the file that we expected.
        self.assertTrue(re.search(r'url\(/static/scss/assets/images_icons-.+\.png\?_=\d+', actual))



import shutil

from django.test import TestCase
from django.core.management import call_command
from django.conf import settings


class CollectStaticTestCase(TestCase):
    @classmethod
    def setUpClass(cls):
        super(CollectStaticTestCase, cls).setUpClass()
        call_command('collectstatic', interactive=False, verbosity=0)


class NoCollectStaticTestCase(TestCase):
    @classmethod
    def setUpClass(cls):
        super(NoCollectStaticTestCase, cls).setUpClass()
        shutil.rmtree(settings.STATIC_ROOT, ignore_errors=True)


def clean_css(string):
    # The output of the compiled CSS doesn't have a newline between the ; and
    # the } for some reason.
    return string.strip() \
        .replace('\n', '') \
        .replace('; ', ';')



from __future__ import absolute_import

from compressor.filters import FilterBase
from compressor.conf import settings

from django_pyscss import DjangoScssCompiler


class DjangoScssFilter(FilterBase):
    compiler = DjangoScssCompiler()

    def __init__(self, content, attrs=None, filter_type=None, filename=None, **kwargs):
        # It looks like there is a bug in django-compressor because it expects
        # us to accept attrs.
        super(DjangoScssFilter, self).__init__(content, filter_type, filename, **kwargs)
        try:
            # this is a link tag which means there is an SCSS file being
            # referenced.
            href = attrs['href']
        except KeyError:
            # this is a style tag which means this is inline SCSS.
            self.filename = None
        else:
            self.filename = href.replace(settings.STATIC_URL, '')

    def input(self, **kwargs):
        return self.compiler.compile_string(self.content, filename=self.filename)



#!/usr/bin/env python
import os
import sys
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "testproject.testproject.settings")

import django
if hasattr(django, 'setup'):
    django.setup()

from django.test.utils import get_runner
from django.conf import settings


def runtests():
    # Stolen from django/core/management/commands/test.py
    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=1, interactive=True)
    # failures = test_runner.run_tests(['tests.test_scss.FindersImportTest.test_relative_import_with_filename'])
    failures = test_runner.run_tests(['tests'])
    sys.exit(bool(failures))



from django.template import Template, Context

from tests.utils import CollectStaticTestCase


APP2_LINK_TAG = """
{% load staticfiles compress %}
{% compress css %}
<link rel="stylesheet" type="text/x-scss" href="{% static 'css/app2.scss' %}">
{% endcompress %}
"""

IMPORT_APP2_STYLE_TAG = """
{% load staticfiles compress %}
{% compress css %}
<style type="text/x-scss">
@import "css/app2.scss";
</style>
{% endcompress %}
"""


class CompressorTest(CollectStaticTestCase):
    def test_compressor_can_compile_scss(self):
        actual = Template(APP2_LINK_TAG).render(Context())
        # 4b368862ec8c is the cache key that compressor gives to the compiled
        # version of app2.scss.
        self.assertIn('4b368862ec8c.css', actual)

    def test_compressor_can_compile_scss_from_style_tag(self):
        actual = Template(IMPORT_APP2_STYLE_TAG).render(Context())
        self.assertIn('4b368862ec8c.css', actual)



#!/usr/bin/env python
from setuptools import setup, find_packages
import os

__doc__ = "Makes it easier to use PySCSS in Django."


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


install_requires = [
    'Django>=1.4',
    'pyScss2',
]

try:
    import pathlib
except ImportError:
    install_requires.append('pathlib')

tests_require = [
    'Pillow',
    'django-compressor>=1.3',
    'django-discover-runner',
    'mock',
]


version = '3.0.0'


setup(
    name='django-pyscss2',
    version=version,
    author="Ivan Kolodyazhny",
    author_email="e0ne@e0ne.info",
    url="https://github.com/e0ne/django-pyscss",
    keywords="django css scss sass pyscss compressor",
    description=__doc__,
    long_description=read('README.rst') + '\n\n' + read('CHANGELOG.rst'),
    packages=[package for package in find_packages() if package.startswith('django_pyscss')],
    install_requires=install_requires,
    tests_require=tests_require,
    zip_safe=False,
    include_package_data=True,
    test_suite='testproject.runtests.runtests',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Topic :: Internet :: WWW/HTTP',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)



from __future__ import absolute_import, unicode_literals

import os

from itertools import product
from pathlib import PurePath

from scss.extension.core import CoreExtension
from scss.source import SourceFile

from ..utils import get_file_and_storage


class DjangoExtension(CoreExtension):
    name = 'django'

    def handle_import(self, name, compilation, rule):
        """
        Re-implementation of the core Sass import mechanism, which looks for
        files using the staticfiles storage and staticfiles finders.
        """
        original_path = PurePath(name)

        search_exts = list(compilation.compiler.dynamic_extensions)
        if original_path.suffix and original_path.suffix in search_exts:
            basename = original_path.stem
        else:
            basename = original_path.name

        if original_path.is_absolute():
            # Remove the beginning slash
            search_path = original_path.relative_to('/').parent
        elif rule.source_file.origin:
            search_path = rule.source_file.origin
            if original_path.parent:
                search_path = os.path.normpath(str(search_path / original_path.parent))
        else:
            search_path = original_path.parent

        for prefix, suffix in product(('_', ''), search_exts):
            filename = PurePath(prefix + basename + suffix)

            full_filename, storage = get_file_and_storage(str(search_path / filename))

            if full_filename:
                with storage.open(full_filename) as f:
                    return SourceFile.from_file(f, origin=search_path, relpath=filename)



from __future__ import absolute_import

import os
from pathlib import PurePath

from django.utils.six.moves import StringIO

from django.conf import settings
from django.contrib.staticfiles.storage import staticfiles_storage

from scss import Compiler, config
from scss.extension.compass import CompassExtension
from scss.source import SourceFile

from .extension.django import DjangoExtension
from .utils import find_all_files, get_file_and_storage


# TODO: It's really gross to modify this global settings variable.
# This is where PyScss is supposed to find the image files for making sprites.
config.STATIC_ROOT = find_all_files
config.STATIC_URL = staticfiles_storage.url('scss/')

# This is where PyScss places the sprite files.
config.ASSETS_ROOT = os.path.join(settings.STATIC_ROOT, 'scss', 'assets')
# PyScss expects a trailing slash.
config.ASSETS_URL = staticfiles_storage.url('scss/assets/')


class DjangoScssCompiler(Compiler):
    def __init__(self, **kwargs):
        kwargs.setdefault('extensions', (DjangoExtension, CompassExtension))
        if not os.path.exists(config.ASSETS_ROOT):
            os.makedirs(config.ASSETS_ROOT)
        super(DjangoScssCompiler, self).__init__(**kwargs)

    def compile(self, *paths):
        compilation = self.make_compilation()
        for path in paths:
            path = PurePath(path)
            if path.is_absolute():
                path = path.relative_to('/')
            filename, storage = get_file_and_storage(str(path))
            with storage.open(filename) as f:
                source = SourceFile.from_file(f, origin=path.parent, relpath=PurePath(path.name))
            compilation.add_source(source)
        return self.call_and_catch_errors(compilation.run)

    def compile_string(self, string, filename=None):
        compilation = self.make_compilation()
        if filename is not None:
            f = StringIO(string)
            filename = PurePath(filename)
            source = SourceFile.from_file(f, origin=filename.parent, relpath=PurePath(filename.name))
        else:
            source = SourceFile.from_string(string)
        compilation.add_source(source)
        return self.call_and_catch_errors(compilation.run)
